/* 
Checksum: 3203539768      (vcgparsa.c) 
*/

/*
 *  This is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *   
 *  GNU VCG is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *   
 *  You should have received a copy of the GNU General Public License
 *  along with GNU Make; see the file COPYING.  If not, write to
 *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. 
 *
 *  Copyright (C) 1994, 1995, 2000, 2004 Free Software Foundation, Inc.
 *
 */


/* lexer****************/

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <math.h>

#include "all.h"

#include "y.tab.h"


extern int yyparse(void);
extern int line_nr;
extern int pos_nr;
extern char *filename;

static void init_hash(void);
static void increase_decode(void);
static int hash(char *string);

/* forward declaration */
void freehash(void);
long hashinsert(char *s);

#ifndef YYLTYPE
typedef struct yyltype {
		int timestamp;
		int first_line;
		int first_column;
		int last_line;
		int last_column;
		char *text;
	}  yyltype;

#define YYLTYPE yyltype
#endif

typedef struct stree_node *syntaxtree;

union  special {
        unsigned char      byte;
        short int          snum;
        unsigned short int usnum;
        int                num;
        unsigned int       unum;
        long int           lnum;
        unsigned long int  ulnum;
        float              realnum;
        double             lrealnum;
        char              *string;
};

struct stree_node {
	int  tag_field;
	int  first_line;
	int  first_column;
	int  last_line;
	int  last_column;
	struct stree_node *father;
	union  special     contents;
	struct stree_node *xson[1];
};

#include "vcgtcodes.h"

/* addon code to lexer/parser */

/************************************************************/


#define hashentrymalloc malloc
#define strmalloc malloc
#define strpmalloc malloc

#define hashentryfree free
#define strpfree free

/* in lexer */
extern void init_lexlex(void);

/* */
void FreeHash(void) { return; }
void init_lex(void) { init_lexlex(); init_hash(); }
long HashInsert(char *s) { return(hashinsert(s)); }
long asciitolong(char *string) { return (strtol(string, (char **)NULL, 10)); }
double asciitodouble(char *string) { return (strtod(string,(char **)NULL)); }

/* */
#define HASH_SIZE 32003
#define HASHTABLE_SIZE 50000L

/*
 * Globals here:
 * void init_lex(void);
 * void deinit_lex(void);
 * long table_size;
 * int hashing_size;
 * void freehash(void);
 * long hashinsert(char *s);
 * long hashtablesize(void);
 * char *de_code(long x);
 */


#ifndef HASH_SIZE
#warning "using default HASH_SIZE -- should not happen -- define HASH_SIZE"
#define HASH_SIZE 211
#endif
#ifndef HASHTABLE_SIZE
#warning "using default HASHTABLE_SIZE -- should not happen -- define HASHTABLE_SIZE"
#define HASHTABLE_SIZE 10000L
#endif

/* We use a hash table with linked lists */

typedef struct hash_node {
        long              num;
        struct hash_node *next;
} *hashentry;

/*--------------------------------------------------------------------*/
/*  Standard Hash Table Routines                                      */
/*--------------------------------------------------------------------*/

/* Global Variables */
/*------------------*/

static char **decode = (char **)0;

/* size of table == HASHTABLE_SIZE */
long table_size = 0L;

/* modulo used for hashing == HASH_SIZE */
int hashing_size = 0;

/*
 *
 */
void init_sym(void)
{
	table_size = HASHTABLE_SIZE;
	hashing_size = HASH_SIZE;
	init_hash(); 
	return;
}

/*
 *
 */
void deinit_sym(void)
{
	freehash();
	return;
}

/*--------------------------------------------------------------------*/
/* Memory allocation for decode table and hash table                  */
/*--------------------------------------------------------------------*/

/* The hash table */

/*
 *
 */
static hashentry *hashtable = (hashentry *)0;

/*
 *
 */
static long hashtop = 1;

/*
 *
 */
static void init_hash(void)
{
	char *h;
	if (!hashtable) {
		/* no hash table present -> allocate table */
		hashtable = (hashentry *)hashentrymalloc((size_t)((HASH_SIZE+1)*sizeof(hashentry)));
		memset(hashtable,0x00,((size_t)((HASH_SIZE+1)*sizeof(hashentry))));
	}
	if (!decode) {
		/* no decode table present -> allocate table */
		hashtop = 1;
		table_size = HASHTABLE_SIZE;
		hashing_size = HASH_SIZE;
		decode   = (char **)strpmalloc((size_t) ((table_size+1L) * sizeof(char *)));
 		memset(decode,0x00,((size_t) ((table_size+1L) * sizeof(char *))));
		/* add 1 fake entry */
		h = strmalloc(strlen("$!+-#Start of Hash#+-!$")+1);
		memset(h,0x00,(strlen("$!+-#Start of Hash#+-!$")+1));
		decode[0] = strcpy(h,"$!+-#Start of Hash#+-!$");
	}
	return;
}

/*
 *
 */
static void increase_decode(void)
{
	char **help;
	char **a;
	char **b;
	long mysize;
	long j;

	/* here: double size of current table size. */
	mysize = table_size * 2L;
	/* help = (char **) calloc((size_t) (mysize+1), sizeof(char *)); */
	help = (char **)strpmalloc((size_t) ((mysize+1)*sizeof(char *)));
	memset(help,0x00,((size_t) ((mysize+1)*sizeof(char *))));
	a = help;
	b = decode;
	for (j=0; j<table_size; j++) 
	{
		*a++ = *b++;
	}
	if (decode) {
		strpfree((char **)decode);
	}
	decode   = help; 
	table_size = mysize;

	return;
}

/*
 *
 */
void freehash(void)
{
	/* FIXME - strings itself to free; see hashinsert() */
	if (decode) {
		strpfree((char **)decode);
	}
	decode   = (char **)0; 
	if (hashtable) {
		/*(void)free((char *)hashtable);*/
    		hashentryfree((hashentry *)hashtable);
	}
	hashtable  = (hashentry *)0;
	return;
}


/*--------------------------------------------------------------------*/
/* Hash table for identifers and strings                              */
/*--------------------------------------------------------------------*/


/* 
 * the hash function 
 */
static int hash(char *string)
{
	int   hashval = 0 ;
	char *s = string;
	/* */
	for(hashval=0; *s != '\0';)
	{
		hashval = (10*hashval - 48 + *s++) % HASH_SIZE;
	}
	hashval = ((hashval<0 ) ? -hashval : hashval);
	return(hashval % HASH_SIZE);
}



/*--------------------------------------------------------------------*/
/* Hash table: visible from outside                                   */
/*--------------------------------------------------------------------*/

long hashinsert(char *s)
{
	hashentry new_node;
	int hashval;
	long look;
	char *h;

	if (!hashtable) {
		init_hash();
	}
	
	hashval  = hash(s);
	new_node = hashtable[hashval];
	look = -1L;
	
	/* see if exixs. -- takes long time -- */
	while (new_node != NULL) {
		if ( strcmp(s, decode[new_node->num]) == 0L) {
			look = new_node->num;
			break;
		}
		new_node=new_node->next;
	}
	if (look == -1L) {
		if (hashtop == table_size-1) {
			increase_decode();
		}
		/* malloc entry structure */
		new_node = (hashentry)hashentrymalloc(sizeof(struct hash_node));
		memset(new_node,0x00,(sizeof(struct hash_node)));
		look = hashtop++;
		new_node->num  = look;
		new_node->next = hashtable[hashval];
		/* malloc string itself */
		h = strmalloc(strlen(s)+1);
		memset(h,0x00,(strlen(s)+1));
		decode[look]    = strcpy(h,s);
		hashtable[hashval] = new_node;
	}
	return(look);
}

/*
 *
 */
long hashtablesize(void)
{

	if (!hashtable) {
		init_hash();
	}

	return(hashtop);
}

/*
 *
 */
char *Decode(long x)
{
	char *ret;
	if (!hashtable) { 
		init_hash(); /* "$!+-#Start of Hash#+-!$" is inserted as fake entry */ 
	}
	if (x<0) {
		ret=(decode[0]);
	} else if (x<hashtop) {
		ret=(decode[x]);
	} else {
		ret=(decode[0]);
	}
	return (ret);
}

/* TODO correct for gcc cpp */
void line_directive(char *text)
{
	char *c,*d;
	c = d = text;
	while ((*c) && (*c!='"')) { c++; }
	while ((*d) && ((*d<'0') || (*d>'9'))) { d++; }
	if (d<c) { line_nr = atoi(d); pos_nr = 0; }
	if (*c) {
		c++;
		d = c;
		while ((*d) && (*d!='"')) { d++; }
		*d=0;
		strcpy(filename, c);
	}
	return;
}



/*
this is used for grepping the prototypes


PROTOTYPEF(char *Decode(long x)); // vcgparsa.c --


*/

/* end. */
