/* 
Checksum: 3686150607      (vcgparsb.c) 
*/

/*
 *  This is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *   
 *  GNU VCG is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *   
 *  You should have received a copy of the GNU General Public License
 *  along with GNU Make; see the file COPYING.  If not, write to
 *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. 
 *
 *  Copyright (C) 1994, 1995, 2000, 2004 Free Software Foundation, Inc.
 *
 */


#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <math.h>

#include "all.h"

#include "y.tab.h"

extern int yyparse(void);

#ifndef YYLTYPE
typedef struct yyltype 
{
		int timestamp;
		int first_line;
		int first_column;
		int last_line;
		int last_column;
		char *text;
}  yyltype;

#define YYLTYPE yyltype
#endif

typedef struct stree_node *syntaxtree;

union  special 
{
	unsigned char      byte;
	short int          snum;
	unsigned short int usnum;
	int                num;
	unsigned int       unum;
	long int           lnum;
	unsigned long int  ulnum;
	float              realnum;
	double             lrealnum;
	char              *string;
};


struct stree_node 
{
	int  tag_field;
	int  first_line;
	int  first_column;
	int  last_line;
	int  last_column;
	struct stree_node *father;
	union  special     contents;
	struct stree_node *xson[1];
};

#include "vcgtcodes.h"

/*********parser********/


/* actual only max 4 elements are used. */
static struct stree_node *treetab[7];

#define parsemalloc ParseMalloc

/* */ 
char * ParseMalloc(int x)
{
	return((char *)malloc(x));
}

/* */
static void parsebcopy (unsigned char *src, unsigned char *dest, int len)
{
	if (dest < src) {
		while (len--) {
			*dest++ = *src++;
		}
	}
	else {
		unsigned char *lasts = src + (len-1);
 		unsigned char *lastd = dest + (len-1);
		while (len--) {
			*(char *)lastd-- = *(char *)lasts--;
    	}
	}
	return;
}

/* global deallocate the complete syntax tree heap */

static union special *stdunion=(union special *)0;

void ParseFree(void)
{
	/* no parseheap used here. just malloced */
#if 0
	syntaxtree help, help2;

	help = parseheap;
	while (help) {
		help2 = *(syntaxtree *)help;
		(void)free((char *)help);
		help = help2;
	}	

	parseheap = (syntaxtree)0;
	stdunion = 0;
#endif
	return;
}

/* alloc node with x sons */
static struct stree_node * st_malloc(int x)
{
	struct stree_node *help=(struct stree_node *)0;
	help = (struct stree_node *) parsemalloc (sizeof(struct stree_node) + x * sizeof(struct stree_node *));
	return ((struct stree_node *) help);
}

static void freeunion(union special *x)
{
	x->string = (char *)stdunion;
	stdunion = x;	
	return;
}

static union special *nextunion(void)
{
	union special *help=(union special *)0;
	if (!stdunion) {
		stdunion = (union special *) parsemalloc (sizeof(union special));
		stdunion->string = 0;
	}
	help = stdunion;
	stdunion = (union special *)stdunion->string;
	return(help);
}

union special *UnionByte(unsigned char x)
{
	union special *help=(union special *)0;
	help = nextunion();
	help->byte = x;
	return(help);
}

union special *UnionLnum(long int x)
{
	union special *help=(union special *)0;
	help = nextunion();
	help->lnum = x;
	return(help);     
}

union special *UnionLrealnum(double x)
{
	union special *help=(union special *)0;
	help = nextunion();
	help->lrealnum = x;
	return(help);     
}

union special *UnionNum(int x)
{
	union special *help=(union special *)0;
	help = nextunion();
	help->num = x;
	return(help);     
}

/*
union special *UnionString(char *x)
{
	union special *help;
	help = nextunion();
	help->string = x;
	return(help);     
}
*/


/* without sons */
syntaxtree BuildCont(int mtag,union special *x,YYLTYPE *l)
{
	syntaxtree help=(syntaxtree)0;
	help = st_malloc(1);
	parsebcopy((unsigned char *)x,(unsigned char *)&help->contents, sizeof(union special));
	freeunion(x);
	tag(help)          = mtag;
	xfirst_line(help)   =l->first_line;
	xfirst_column(help) =l->first_column;
	xlast_line(help)    =l->last_line;
	xlast_column(help)  =l->last_column;
	xfather(help)       =(syntaxtree)0;
	son1(help) = (syntaxtree)0; 
	return(help);
}

#define ANSI_C

/* with sons */
syntaxtree BuildTree(int mtag,int len,union special *x,YYLTYPE *l, ...)
{
	int i=0;
	int j=0;
	va_list pvar;
	syntaxtree help=(syntaxtree)0;

#ifndef ANSI_C
	int mtag;
	int len;
	union special *x;
	YYLTYPE *l;
	va_start(pvar);
#endif

#ifdef ANSI_C
	va_start(pvar,l);
#endif

#ifndef ANSI_C
	mtag = va_arg(pvar, int);
	len = va_arg(pvar, int);
	x   = va_arg(pvar, union special *);
	l   = va_arg(pvar, YYLTYPE *);
#endif
 
	i = 0;
	help = va_arg(pvar, syntaxtree);
	while (i < len) {
		treetab[i++] = help;
		help = va_arg(pvar, syntaxtree);
	}
	va_end(pvar);

	help = st_malloc((i<1?1:i));
	parsebcopy((unsigned char *)x,(unsigned char *)&help->contents, sizeof(union special));
	freeunion(x);
	tag(help)          = mtag;
	xfirst_line(help)   =l->first_line;
	xfirst_column(help) =l->first_column;
	xlast_line(help)    =l->last_line;
	xlast_column(help)  =l->last_column;
	xfather(help)       =(syntaxtree)0;

	for (j=1; j<=i; j++) 
	{
		son(help,j) = treetab[j-1]; 
		if (treetab[j-1]!=(syntaxtree)0) {
			xfather(treetab[j-1]) = help;
		}
	}
	return(help);
}

/* helper */
struct stree_node * revertsyntaxtree(struct stree_node * list)
{
	struct stree_node *last=(struct stree_node *)0;
	struct stree_node *act=(struct stree_node *)0;
	struct stree_node *next=(struct stree_node *)0;
	act  = list;
	while (act) {
		next = son2(act);
		son2(act) = last;
		if (last) { xfather(last)=act; }
		last = act;	
		act = next;
	}
	if (last) { xfather(last)=(struct stree_node *)0; }
	return(last);
}

/* */
static const int yyconst_arity[] = {
#define VCG_T_Co(a,b,c) c,
#include "vcgtcode.h"
};

/* */
int ConstructorArity(int i)
{
	return(yyconst_arity[i]);
}


/* */
struct stree_node * Syntax_Tree;

extern int nr_errors;
extern int line_nr;
extern int pos_nr;
extern char *filename;
extern void init_lexlex(void);

void yyerror(char *s) { printf("yyerror(): '%s' file '%s' line %d position %d\n",s,filename,line_nr+1,pos_nr); fflush(stdout); return; }

/*   returns the number of detected errors and binds the syntaxtree
 *   to the variable syntaxtree.
 */
int parse(void)
{
	nr_errors=0;
	line_nr = 1;
	pos_nr = 0;
	init_lexlex();
	yyparse();
	return(nr_errors);
}

/* */
long long_atoi(char *c)
{
	long res, sign;

	res = 0;
	if (!c) return(0L);
	if (*c=='-')  { sign = -1L; c++; }
	else sign = 1L;
	while (*c) {
		switch (*c) {
		case '0': res = res * 10L;      break;
		case '1': res = res * 10L + 1L; break;
		case '2': res = res * 10L + 2L; break;
		case '3': res = res * 10L + 3L; break;
		case '4': res = res * 10L + 4L; break;
		case '5': res = res * 10L + 5L; break;
		case '6': res = res * 10L + 6L; break;
		case '7': res = res * 10L + 7L; break;
		case '8': res = res * 10L + 8L; break;
		case '9': res = res * 10L + 9L; break;
		default: return(sign * res);
		}
		c++;
	}
	return(sign * res);
}

/*  Translate escape sequences in strings. The result remains
 *  in text, because the result is shorter !
 *
 *   \\ -> \        \" -> "     \n -> CR    
 *
 *  et cetera.
 */

void escape_transl(char *text)
{
 	char *c;
	char *d;
	int i;

	c = d = text;
	while (*c) {
		if (*c == '\\') {
			c++;
			switch (*c) {
#ifdef ANSI_C 
			/* do we really want 'beeps' from a compiler graph? */
			case 'a' : *d++ = '\a'; break;
#endif
			case 'b' : *d++ = '\b'; break;
			case 'f' : *d++ = '\f'; break;
			case 'n' : *d++ = '\n'; break;
			case 'r' : *d++ = '\r'; break;
			case 't' : *d++ = '\t'; break;
			case 'v' : *d++ = '\v'; break;
			case 'x' :
				   c++;
				   i = 0;
				   while (  ((*c>='0')&&(*c<='9')) 
					  ||((*c>='A')&&(*c<='F'))
					  ||((*c>='a')&&(*c<='f')) ) {
					if ((*c>='0')&&(*c<='9')) 
						i = i*16 + (*c)-'0'; 
					else if ((*c>='A')&&(*c<='F')) 
						i = i*16 + (*c)-'A'+10; 
					else if ((*c>='a')&&(*c<='f')) 
						i = i*16 + (*c)-'a'+10; 
					c++;
				   }
				   c--;
				   *d++ = i;
				   break;
			case '0' :
			case '1' :
			case '2' :
			case '3' :
			case '4' :
			case '5' :
			case '6' :
			case '7' :
				  i = (*c) - '0';
				  c++;
				  if ((*c>='0') && (*c<='7')) {
					i = i*8 +(*c)-'0';
				  	c++;
				  	if ((*c>='0') && (*c<='7')) {
						i = i*8 +(*c)-'0';
						c++;
					}
				  }
				  c--;
				  *d++ = i;
				  break;
			default : *d++ = *c; break;
			}
		}
		else *d++ = *c;
		c++;
	}
	*d = 0;
}



/*
this is used for grepping the prototypes


PROTOTYPEF(void escape_transl(char *text)); // vcgparsb.c --
PROTOTYPEF(int ConstructorArity(int i)); // vcgparsb.c --


*/

/* end. */
